#ifndef COUPMATU_H
#define COUPMATU_H

#include <stdio.h>
#include "alias.h"
#include "aliasc.h"
#include "intpointsc.h"
#include "isotrmatc.h"
#include "sejtkrmatc.h"
#include "baroghelBc.h"
#include "concreteBc.h"
#include "C60baroghelc.h"
#include "C30baroghelc.h"
#include "o30bazantc.h"
#include "C60bazantc.h"
#include "glasgowmatc.h"
#include "glasgowcoup.h"
#include "consol_awf1c.h"
#include "consol_awf2c.h"
#include "xfile.h"

class coupmatu
{
 public:
  coupmatu (void);
  ~coupmatu (void);
  void ipalloc (void);
  long intpnum (void);
  void readip (XFILE *in);
  void readmatchar (XFILE *in);
  void intpointalloc ();
  void intpointinit ();
  void initmaterialmodels ();
  void initvalues (long ipp,long im,long ido);
  void updateipval (void);
  void updateipvalmat (long ipp,long im,long ido);
  void compute_tempstrains (void);
  void compute_tempstrainsmat (long ipp,long im,long ido);
  void compute_eigstrains (void);
  void compute_eigstrainsmat (long ipp,long im,long ido);
  void alloceigstrains ();
  void alloceigstresses ();
  void alloctempstrains ();
  void nulltempstrains ();
  void nulleigstrains ();
  void eigstrmod ();
  void read (XFILE *in);
  void matstiff (matrix &d,long ipp);
  void matcond (matrix &d,long ipp,long ri,long ci);
  void matcap (matrix &d,long ipp,long ri,long ci);
  void volume_rhs1 (matrix &d,long ipp,long ri,long ci);
  void volume_rhs2 (matrix &d,long ipp,long ri,long ci,long ncomp);

  void computenlstresses (matrix &d,long ipp);
  void storestresses_cmu (long ipp,long fi,vector &gr);
  void givestresses_cmu (long ipp,long fi,vector &fl);
  void storestrain_cmu (long ipp,long fi,vector &eps);
  void givestrain_cmu (long ipp,long fi,vector &eps);
  void storegrad_cmu (long lcid,long ipp,vector &gr);
  void givegrad_cmu (long lcid,long ipp,vector &gr);
  void storeflux_cmu (long lcid,long ipp,vector &fl);
  void givefluxes_cmu (long lcid,long ipp,vector &fl);

  //for testing
  void computeigenstresses (matrix &d,long ipp,vector &fl);

  void material_matrix (matrix &d,long eid,long ri,long ci,strastrestate ssst);

  //  number of material types
  long nmt;
  //  total number of all integration points
  long tnip;
  //  pointer to integration points
  intpointsc *ip;

  //  COUPLING PROBLEMS
  //  one-phase medium - heat transfer - isotropic heat-elast material
  isotrmatc *itrmc;
  //  saturated-unsaturated one-phase flow in deforming meduim
  sejtkrmatc *sejtkrmc;
  con_awf1matc *consol_awf1c;
  //two-phase flow
  con_awf2matc *consol_awf2c;
  //three-phase flow
  baroghelmatc *baroghelc;
  concreteBmatc *concretec;
  C60barmatc  *C60baroghelc;
  C30barmatc  *C30baroghelc;
  o30bazmatc  *o30bazantc;
  C60bazmatc  *C60bazantc;
  glasgowmatc *tenchc;
  glasgowcoup *gcm;

  ///  array containing eigenstrains
  ///  the array is allocated only for several problems
  ///  eigstrains[number of int. point][number of component]
  ///  the array is allocated in function mechbclc::read
  double **eigstrains;
  ///  array containing eigenstresses
  ///  the array is allocated only for several problems
  ///  eigstresses[number of int. point][number of component]
  double **eigstresses;
  ///  array containing strains caused by temperature
  ///  tempstrains[number of int. point][number of component]
  double **tempstrains;
    

  ///  array containing temperatures at integration points
  ///  tempr[number of integ. point] - temperature at the required integration point
  double *tempr;
  ///  array containing initial temperatures at integration points
  ///  inittempr[number of integ. point] - initial temperature at the required integration point
  double *inittempr;
  ///  array containing non-mechanical and non-transport quantities used for coupled problems
  ///  nonmechtransq[number of integ. point][quantity no.] - nonmechtransport quantity at the required integration point
  double **nonmechtransq;
  ///  number of nontransport quantities
  long nmtrq;
};

#endif
