#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include "couptop.h"
#include "globalc.h"
#include "gtopology.h"
#include "element.h"
#include "elementc.h"


couptop::couptop (void)
{
  ne=0;
  elements=NULL;
}

couptop::~couptop (void)
{
  delete [] elements;
}

void couptop::read (XFILE *in)
{
  long i,j;
  
  if ((Cp->tprob==par_coupl_mech_trans) || (Cp->tprob==growing_par_coupl_mech_trans)){
    
  }

  if (Cp->tprob==fully_coupled_mech_trans){
    //  element data
    xfscanf (in,"%ld",&ne);
    
    if (Mesprt==1)  fprintf (stdout,"\n number of elements  %ld",ne);
    elements = new elementc [ne];
    
    for (i=0;i<ne;i++){
      j=ne+1;
      xfscanf (in,"%ld",&j);
      if (j<1)
	fprintf (stderr,"\n\n element number in function couptop::read is less than 1.\n");
      if (j>ne){
	fprintf (stderr,"\n\n element number in function couptop::read is");
	fprintf (stderr,"\n greater than total number of elements.\n");
      }
      elements[j-1].read (in);
    }
  }
}

long couptop::mesh_check(void)
{
  long i,j,err=0;
  double fabx,faby,fabz;

  //nodes check
  if(Mt->nn != Tt->nn){
    err = 1;
    fprintf (stdout,"\n\n Different number of nodes in mefel and trfel input file \n\n");
    return err;
  }

  for(i=0;i<Mt->nn;i++){
    fabx = Gtm->gnodes[i].x - Gtt->gnodes[i].x;  
    faby = Gtm->gnodes[i].y - Gtt->gnodes[i].y;  
    fabz = Gtm->gnodes[i].z - Gtt->gnodes[i].z;  
    
    if(fabs(fabx) >= 1.0e-4){
      err = 2;
      fprintf (stdout,"\n\n Different x coordinates in node number %ld\n\n",i+1);
      return err;
    }
    if(fabs(faby) >= 1.0e-4){
      err = 3;
      fprintf (stdout,"\n\n Different y coordinates in node number %ld\n\n",i+1);
      return err;
    }
    if(fabs(fabz) >= 1.0e-4){
      err = 4;
      fprintf (stdout,"\n\n Different z coordinates in node number %ld\n\n",i+1);
      return err;
    }
  } 
  
  //elems check
  for(i=0;i<Tt->ne;i++){
    for(j=0;j<Gtt->gelements[i].nne;j++){
      if((Gtt->gelements[i].nodes[j]) != (Gtm->gelements[i].nodes[j])){
	fprintf (stdout,"\n\n Different node conectivities in element number %ld, node number %ld",i+1,j+1);
	fprintf (stdout,"\n Trfel node number %ld, Mefel node number %ld\n\n",Gtt->gelements[i].nodes[j]+1,Gtm->gelements[i].nodes[j]+1);
	err = 5;
	return err;
      }
    }
  }
  
  return err;
}



/**
   function returns element type
   
   @param eid - element id
*/
elemtypec couptop::give_elem_type (long eid)
{
  return (elements[eid].te);
}

/**
   function returns number of DOFs of node
   
   @param nid - node id
*/
long couptop::give_ndofn (long nid)
{
  return Gtu->give_ndofn (nid);
}

/**
   function returns number of DOFs of element
   
   @param eid - element id
   
   JK, 22.2.2002
*/
long couptop::give_ndofe (long eid)
{
  long ndofe;
  elemtypec te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case coupbar:{         ndofe=Cbar->tndofe;         break;  }
  case coupquad:{        ndofe=Cquad->tndofe;        break;  }
  case coupaxiquad:{     ndofe=Caxiq->tndofe;        break;  }
  case couphex:{         ndofe=Chex->tndofe;         break;  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n transtop::give_ndofe (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return ndofe;
}

/*
long couptop::give_nne (long eid)
{
  long nne;
  elemtypec te;
  
  te = Ct->give_elem_type (eid);
  nne = Ct->give_nne_inner (te);
  return nne;
}
*/


/**
   function returns total number of upper integration points on one element
   
   @param eid - element id
   
   07/05/2010 TKr
*/
long couptop::give_upper_tnip (long eid)
{
  long nip;
  elemtypec te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case coupbar:{               nip=Cbar->tnipu;     break;  }
  case coupquad:{              nip=Cquad->tnipu;    break;  }
  case coupaxiquad:{           nip=Caxiq->tnipu;    break;  }
  case couphex:{               nip=Chex->tnipu;     break;  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n couptop::give_upper_nip (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return nip;
}


/**
   function returns total number of lower integration points on one element
   
   @param eid - element id
   
   07/05/2010 TKr
*/
long couptop::give_lower_tnip (long eid)
{
  long nip;
  elemtypec te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case coupbar:{               nip=Cbar->tnipl;     break;  }
  case coupquad:{              nip=Cquad->tnipl;    break;  }
  case coupaxiquad:{           nip=Caxiq->tnipl;    break;  }
  case couphex:{               nip=Chex->tnipl;     break;  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n couptop::give_upper_nip (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return nip;
}



/**
   function returns number of integration points
   
   @param eid - element id
   @param ri,ci - row and column indices
   
   30.12.2001
*/
long couptop::give_upper_nip (long eid,long ri,long ci)
{
  long nip;
  elemtypec te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case coupbar:{               nip=Cbar->nipu[ri][ci];     break;  }
  case coupquad:{              nip=Cquad->nipu[ri][ci];    break;  }
  case coupaxiquad:{           nip=Caxiq->nipu[ri][ci];    break;  }
  case couphex:{               nip=Chex->nipu[ri][ci];     break;  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n couptop::give_upper_nip (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return nip;
}

/**
   function returns number of integration points
   
   @param eid - element id
   @param ri,ci - row and column indices
   
   30.12.2001
*/
long couptop::give_lower_nip (long eid,long ri,long ci)
{
  long nip;
  elemtypec te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case coupbar:{               nip=Cbar->nipl[ri][ci];     break;  }
  case coupquad:{              nip=Cquad->nipl[ri][ci];    break;  }
  case coupaxiquad:{           nip=Caxiq->nipl[ri][ci];    break;  }
  case couphex:{               nip=Chex->nipl[ri][ci];     break;  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function");
    fprintf (stderr,"\n couptop::give_lower_nip (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return nip;
}

/**
   function returns numbers of nodes defining apropriate element
   
   @param eid - element id
   @param nodes - array containing nodes
   
   22.7.2001
*/
void couptop::give_elemnodes (long eid,ivector &nodes)
{
  Gtu->give_nodes (eid,nodes);
}

/**
   function extracts all code numbers of actual element
   
   @param eid - element id
   @param cn - array containing all code numbers on element
   
   25.6.2001
*/
void couptop::give_code_numbers (long eid,long *cn)
{
  Gtu->give_code_numbers (eid,cn);
}

/**
   function returns node coordinates of the appropriate element
   
   @param x,y - vectors containing node coordinates
   @param eid - element id
   
   19.7.2001
*/
void couptop::give_node_coord2d (vector &x,vector &y,long eid)
{
  Gtu->give_node_coord2d (x,y,eid);
}

/**
   function returns node coordinates of the appropriate element
   
   @param x,z - vectors containing coordinates
   @param eid - element id
   
   19.7.2001
*/
void couptop::give_node_coord2dxz (vector &x,vector &z,long eid)
{
  Gtu->give_node_coord2dxz (x,z,eid);
}

/**
   function returns node coordinates of the appropriate element
   
   @param x,y,z - vectors containing coordinates
   @param eid - element id
   
   19.7.2001
*/
void couptop::give_node_coord3d (vector &x,vector &y,vector &z,long eid)
{
  Gtu->give_node_coord3d (x,y,z,eid);
}

/**
   function returns number of blocks in characteristic matrices
   
   @param eid - element id
   
   8.5.2002
*/
long couptop::give_mnb (long eid)
{
  long mnb;
  elemtypec te;
  
  te = give_elem_type (eid);
  
  switch (te){

  case coupbar:{      mnb=Cbar->mnb;    break;  }
  case coupquad:{     mnb=Cquad->mnb;   break;  }
  case coupaxiquad:{  mnb=Caxiq->mnb;   break;  }
  case couphex:{      mnb=Chex->mnb;    break;  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function give_nb (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return mnb;
}

/**
   function returns number nodes on elements
   
   @param te - element type
   
   27.2.2002
*/
/*
long couptop::give_nne_inner (elemtypec te)
{
  long nne;
  switch (te){
  case coupbar:{      nne=Cbar->nne;     break;  }
  case coupquad:{    nne=Cquad->nne;   break;  }
  case coupaxiquad:{  nne=Caxiq->nne;    break;  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function give_nne_inner (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return nne;
}
*/

/**
   function returns number of strain/stress components
   
   @param eid - element id
   
   30.12.2001
*/
long couptop::give_mncomp (long eid)
{
  long mncomp;
  elemtypec te;
  
  te = give_elem_type (eid);
  
  switch (te){
  case coupbar:{       mncomp=Cbar->tnmcomp;     break;  }
  case coupquad:{      mncomp=Cquad->tnmcomp;    break;  }
  case coupaxiquad:{   mncomp=Caxiq->tnmcomp;    break;  }
  case couphex:{       mncomp=Chex->tnmcomp;     break;  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function give_ncomp (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return mncomp;
}

/*
strastrestate couptop::give_mssst (long eid,long bi)
{
  strastrestate mssst;
  elemtypec te;
  
  te = give_elem_type (eid);

  switch (te){
  case coupbar:{      mssst = Cbar->mssst;         break;  }
  case coupquad:{    mssst = Cquad->mssst;       break;  }
  case coupaxiquad:{  mssst = Caxiq->mssst;        break;  }
  default:{
    fprintf (stderr,"\n\n unknown element type is required in function give_ssst (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return mssst;
}
*/

strastrestate couptop::give_mssst (long eid,long bi)
{
  strastrestate mssst;
  //mssst = Mt->elements[eid].ssst;
  mssst = Mt->give_ssst (eid,bi);
  return mssst;
}
