#include <math.h>
#include <string.h>
#include "fcsolver.h"
#include "globalc.h"
#include "global.h"
#include "globalt.h"
#include "globmat.h"
#include "globmatt.h"
#include "globmatc.h"
#include "gmatrix.h"
#include "gtopology.h"
#include "elemswitch.h"
#include "dloadcase.h"
#include "transprint.h"
#include "mechprint.h"
#include "backupsol.h"
#include "backupsolt.h"


/**
   function solves fully coupled thermo-hydro-mechanical problem
   
   JK
*/
void solve_fcouplprob ()
{
  //------------------------------------
  //myslim, ze toto tady byt nemusi TKr 12/9/2008
  double *rhs;
  
  Lsrs->lhs=Lsrsc->lhs;
  Lsrs->rhs=Lsrsc->rhs;
  Lsrst->lhs=Lsrsc->lhs;
  Lsrst->rhs=Lsrsc->rhs;
  
  rhs = Lsrsc->rhs;
  
  //  vectors of loading
  // zde musi byt dlc a sesteveni pro pocatecni stav
  //Mb->dlc[0].assemble (0,rhs,NULL,Ndofc,Cp->timecon.starttime ());
  //Mb->lc[1].assemble (0,rhs+Ndofc);
  //-------------------------------------


  //  solver
  nonlinear_solver_coupl (0);
  
  Lsrs->lhs=NULL;
  Lsrs->rhs=NULL;
  Lsrst->lhs=NULL;
  Lsrst->rhs=NULL;
}


void nonlinear_solver_coupl (long lcid)
{
  switch (Cp->tnlinsol){
  case newtonc:{
    //newton_raphson_coupl (lcid);
    newton_raphson_coupl_new (lcid);
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown solver of nonlinear equation system is required");
    fprintf (stderr,"\n in function nonlinear_solver (file %s, line %d).\n",__FILE__,__LINE__);
  }
  }

}


/**
   function solves fully coupled thermo-hydro-mechanical time-dependent problem
   by the Newton-Raphson method
   
   @param lcid - load case id
   
   24/4/2007, revised by TKr 12/9/2008
*/
void newton_raphson_coupl_new (long lcid)
{
  long i,j,k,n,ini;
  double zero,dt,end_time,alpha,norfb,err;
  double *d,*fb,*fi,*p,*lhs,*lhsi,*tdlhs,*rhs,*arhs;
  double *fp,*fl;
  
  //  coefficient of trapezoidal method
  alpha=Cp->alpha;
  zero=Cp->zero;
  //  maximum number of iterations in inner loop
  ini=Cp->niilnr;
  //  required norm of vector of unbalanced forces and fluxes
  err=Cp->errnr;

  //  total number of degrees of freedom of problem
  //  it is sum of DOFs from MEFEL and TRFEL
  n=Ndofc;
  
  //  pointers assigment to local arrays
  //  arrays are allocated in METR in Lsrsc

  //  vector of nodal values
  lhs = Lsrsc->give_lhs (lcid);
  //  vector of time derivatives of nodal values
  tdlhs = Lsrsc->give_tdlhs (lcid);
  //  vector of initial values
  lhsi = Lsrsc->lhsi;
  //  vector of the right hand side
  rhs = Lsrsc->give_rhs (lcid);
  
  //  initial values of arryas
  nullv (lhs,n);
  nullv (tdlhs,n);
  nullv (rhs,n);

  //-----------------------------------
  //this below should be changed??!!
  //  pointers assigment to MEFEL part
  //  arrays are allocated in METR
  Lsrs->lhs = lhs;
  Lsrs->rhs = rhs;
  
  //  pointers assigment to TRFEL part
  //  arrays are allocated in METR
  Lsrst->lhs = lhs;
  Lsrst->tdlhs = tdlhs;
  Lsrst->rhs = rhs;
  //-----------------------------------

  //  array containing predictors
  d = new double [n];
  nullv (d,n);
  //  auxiliary vector
  p = new double [n];
  nullv (p,n);
  //....
  arhs = new double [n];
  nullv (arhs,n);
  //  vector of prescribed forces from the previous step - overit??!!
  fp = new double [n];
  nullv (fp,n);
  //....
  fb = new double [n];
  nullv (fb,n);
  //....
  fi = new double [n];
  nullv (fi,n);
  fl = new double [n];
  nullv (fi,n);
  
  //driving time from METR
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
  //  mechanical time controller is rewritten by coupled time controller
  Mp->time = Cp->time;
  Mp->timecon.take_values (Cp->timecon);
  //  transport time controller is rewritten by coupled time controller
  Tp->time = Cp->time;
  Tp->timecont.take_values (Cp->timecon);
  

  //  nodes - integration points interpolation
  approximationc ();  

  //  initiation of coupled material models
  Cmu->initmaterialmodels();  
  //Cml->initmaterialmodels();  

  /**************************/
  /*  main iteration loop  **/
  /**************************/
  i=0;
  //  inital printing of output and graphical informations
  if (Mp->hdbcont.restore_stat()){
    if (Mespr==1)
      fprintf (stdout,"\n Reading of backup file for MEFEL\n");
    //solver_restore (lhs,fp,i,Mp->time,dt,&Mp->timecon,Ndofm);//overit??!!
    //  initiation of mechanical material models
    //  approximation of temperatures and humidities from TRFEL nodes to MEFEL integration points
    init_trfel_mefel ();
    Mm->initmaterialmodels(lcid);
    print_init(-1, "at");
  }
  else
  {
    // initiation of mechanical material models
    //  approximation of temperatures and humidities from TRFEL nodes to MEFEL integration points
    init_trfel_mefel ();
    Mm->initmaterialmodels(lcid);
    // compute stresses from eigenstrains or initial irreversible strains
    if (Mp->eigstrains > 0)
      internal_forces(lcid, fp);
    print_init(-1, "wt");
    print_step(lcid, i, Cp->time, rhs);
    print_flush();
  }
  if (Tp->hdbcont.restore_stat()){
    if (Mesprt==1)
      fprintf (stdout,"\n Reading of backup file for TRFEL\n");
    //npsolvert_restore (lhs,tdlhs,rhs,i,Cp->time,dt,Cp->timecon,Ndoft);//overit??!!
    //  initiation of transport material models
    Tm->initmaterialmodels();
    compute_req_valt (0);
    print_initt(-1, "at");
  }
  else{
    //  initiation of transport material models
    Tm->initmaterialmodels();
    compute_req_valt (0);
    print_initt(-1, "wt");
    print_stept(0,i,Tp->time,NULL);
    print_flusht();
  }

  do{
    
    //  new time and new time increment
    Cp->time = Cp->timecon.newtime ();
    dt = Cp->timecon.actualbacktimeincr ();
    Mp->time = Cp->time;
    Mp->timecon.take_values (Cp->timecon);
    Tp->time = Cp->time;
    Tp->timecont.take_values (Cp->timecon);
    
    if (Mesprc==1)  fprintf (stdout,"\n\n -------------------------------------------------------------------------------------------");
    if (Mesprc==1)  fprintf (stdout,"\n NEW METR TIME STEP = %ld: METR TIME = %e, metr time increment = %e",i,Cp->time,dt);
    if (Mesprc==1)  fprintf (stdout,"\n -------------------------------------------------------------------------------------------");
    
    //updating of int. points material models and approximations
    approximationc ();
    //updating of material data
    updateval();
    //copying of data
    copy_data();

    i++;

    //  stiffness and conductivity matrices
    zero_order_matrix (lcid);

    //  capacity matrix
    first_order_matrix (lcid);
    
    //  predictor   d+(1-alpha)*dt*v
    for (j=0;j<n;j++){
      d[j] = lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  assembling of vectors of prescribed nodal forces and fluxes
    metr_right_hand_side (lcid,rhs,fl);

    //  auxiliary vector  K (d+(1-alpha)*dt*v)
    D0mat->gmxv (d,p);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    D0mat->scalgm (dt*alpha);
    D0mat->addgm (1.0,*D1mat);
    D0mat->copygm (*D1mat);

    //  increments of prescribed forces
    //incr_internal_gforces (lcid,fi); tady promyslet??!! 7/6/2012

    //  right hand side assembling
    //  F - K (d+(1-alpha)*dt*v)
    for (j=0;j<n;j++){
      fb[j] = rhs[j]-p[j];//+fi[j];
    }
    
    //  solution of the system of algebraic equations
    Cp->ssle->solve_system (Gtu,D1mat,tdlhs,fb,Outc);

    //  new lef hand side vector (unknowns) vector
    //  d_{i+1}=\tilda{d_{i+1}} + alpha.dt.v_{i+1}
    for (j=0;j<n;j++){
      lhs[j] = d[j] + alpha*dt*tdlhs[j];
    }

    //  inner iteration loop
    for (j=0;j<ini;j++){
      
      solution_correction ();
      //updating of int. points material models and approximations
      approximationc ();
      //updating of material data
      updateval();
      //copying of data
      copy_data();
      
      if (Cp->fcsolv == fullnewtonc){
	// matrices are computing in each inner iteration
	//  cleaning of the conductivity matrix
	if (D0mat != NULL){
	  delete D0mat;
	  D0mat=NULL;
	}
	//  cleaning of the capacity matrix
	if (D1mat != NULL){
	  delete D1mat;
	  D1mat=NULL;
	}
	
	//  stiffness and conductivity matrices
	zero_order_matrix (lcid);
	
	//  capacity matrix
	first_order_matrix (lcid);
	
	//  auxiliary vector  K (d+(1-alpha)*dt*v)
	D0mat->gmxv (d,p);
	
	//  matrix of the system of equations
	//  C + alpha.dt.K
	D0mat->scalgm (dt*alpha);
	D0mat->addgm (1.0,*D1mat);
	D0mat->copygm (*D1mat);
      }
      
      //  vector of unbalanced fluxes and forces
      if (Cp->restype == fluxesc){
	// Solver computes unbalanced fluxes and forces
	//  computation of internal forces
	internal_gforces (lcid,fi);//tady promyslet??!!
	for (k=0;k<n;k++){
	  fb[k] = fl[k] - fi[k];//tady promyslet??!!
	}
      }
      if (Cp->restype == lrhsc){
	//correct right hand side
	metr_right_hand_side (lcid,rhs,fl);
	// Solver computes residuum from system of equations
	D0mat->gmxv (tdlhs,fp);
	//  computation of internal forces
	internal_gforces (lcid,fi);//tady promyslet??!!
	// Res. = F - K (d+(1-alpha)*dt*v) - (C + alpha*dt*K)*v 
	for (k=0;k<n;k++){
	  fb[k] = rhs[k] - p[k] - fp[k];//tady promyslet??!!
	}
      }
      
      norfb = ss (fb,fb,n);
      
      //if (Mesprc==1)
      fprintf (stdout,"\n Inner loop j =  %ld     norf = %e\n",j,norfb);
      
      if (norfb<err) break;
      
      Cp->ssle->solve_system (Gtu,D1mat,p,fb,Outc);
      
      for (k=0;k<n;k++){
	tdlhs[k]+=p[k];
	lhs[k]+=alpha*dt*p[k];
      }
    }
    
    compute_req_val (0);
    print_step(0,i,Cp->time,NULL);
    print_flush();
    compute_req_valt (0);
    print_stept(0,i,Cp->time,NULL);
    print_flusht();
    
  }while(Cp->time<end_time);
  
  print_close ();
  print_closet ();
  
  delete [] fp;
  delete [] fl;
  delete [] fi;
  delete [] fb;
  delete [] p;
  delete [] arhs;
  delete [] d;
}


/**
   function solves system of nonlinear algebraic
   equations by Newton-Raphson method
   
   @param lcid - load case id
   
   5.4.2003
*/
void newton_raphson_coupl (long lcid)
{
  long i,j,k,n,ini;
  double zero,dt,end_time,alpha,norfb,err;
  double *d,*fb,*fi,*p,*lhs,*lhsi,*tdlhs,*rhs,*arhs;
  
  //  coefficient of trapezoidal method
  alpha=Cp->alpha;
  zero=Cp->zero;
  //  maximum number of iterations in inner loop
  ini=Cp->niilnr;
  //  required norm of vector of unbalanced forces and fluxes
  err=Cp->errnr;

  //  total number of degrees of freedom of problem
  //  it is sum of DOFs from MEFEL and TRFEL
  n=Ndofc;
  
  //  pointers assigment to local arrays
  //  arrays are allocated in METR in Lsrsc

  //  vector of nodal values
  lhs = Lsrsc->give_lhs (lcid);
  //  vector of time derivatives of nodal values
  tdlhs = Lsrsc->give_tdlhs (lcid);
  //  vector of initial values
  lhsi = Lsrsc->lhsi;
  //  vector of the right hand side
  rhs = Lsrsc->give_rhs (lcid);
  
  //  initial values of arryas
  nullv (lhs,n);
  nullv (tdlhs,n);
  nullv (rhs,n);

  //-----------------------------------
  //this below should be changed??!!
  //  pointers assigment to MEFEL part
  //  arrays are allocated in METR
  Lsrs->lhs = lhs;
  Lsrs->rhs = rhs;
  
  //  pointers assigment to TRFEL part
  //  arrays are allocated in METR
  Lsrst->lhs = lhs;
  Lsrst->tdlhs = tdlhs;
  Lsrst->rhs = rhs;
  //-----------------------------------

  //  array containing predictors
  d = new double [n];
  nullv (d,n);
  //  auxiliary vector
  p = new double [n];
  nullv (p,n);
  //....
  arhs = new double [n];
  nullv (arhs,n);
  //....
  fb = new double [n];
  nullv (fb,n);
  //....
  fi = new double [n];
  nullv (fi,n);
  
  //  starting time
  Cp->time=Cp->timecon.starttime ();
  //  time increment
  dt=Cp->timecon.initialtimeincr ();
  //  end time
  end_time = Cp->timecon.endtime ();
  
  //  nodes - integration points interpolation
  approximationc ();  
  
  //  initiation of coupled material models
  //Cm->initmaterialmodels();  
  //  initiation of mechanical material models
  //Mm->initmaterialmodels();
  //  initiation of transport material models
  //Tm->initmaterialmodels();

  //computation of required values in MEFEL
  compute_req_valt (0);

  /**************************/
  /*  main iteration loop  **/
  /**************************/
  i=0;
  print_init(-1, "wt");
  print_step(0,i,Cp->time,NULL);
  print_initt(-1, "wt");
  print_stept(0,i,Cp->time,NULL);
  do{
    
    if (Mesprc==1)  fprintf (stdout,"\n time %e",Cp->time);
    
    //  stiffness and conductivity matrices
    zero_order_matrix (lcid);

    //  capacity matrix
    first_order_matrix (lcid);
    


    //fprintf (Outc,"\n\n\n MATICE VODIVOSTI (TUHOSTI) \n\n\n");
    //D0mat->printmat (Outc);

    //fprintf (Outc,"\n\n\n MATICE KAPACITY \n\n\n");
    //D1mat->printmat (Outc);

    //  predictor   d+(1-alpha)*dt*v
    for (j=0;j<n;j++){
      d[j] = lhs[j] + (1.0-alpha)*dt*tdlhs[j];
    }
    
    //  right hand side
    metr_right_hand_side (lcid,rhs,p);

    //  auxiliary vector  K (d+(1-alpha)*dt*v)
    D0mat->gmxv (d,p);
    
    //  matrix of the system of equations
    //  C + alpha.dt.K
    D0mat->scalgm (dt*alpha);
    D0mat->addgm (1.0,*D1mat);
    
    //  F - K (d+(1-alpha)*dt*v)
    for (j=0;j<n;j++){
      fb[j] = rhs[j] - p[j];
    }
    
    //fprintf (Outc,"\n\n\n\n\n kontrola prave strany");
    //for (ijk=0;ijk<n;ijk++){
    //fprintf (Outc,"\n %ld %e %e %e",ijk,fb[ijk],rhs[ijk],p[ijk]);
    //}

    //  solution of the system of algebraic equations
    //D0mat->solve_system (Gtt,tdlhs,fb);
    Cp->ssle->solve_system (Gtt,D0mat,tdlhs,fb,Outt);

    //fprintf (Outc,"\n\n\n\n\n kontrola reseni");
    //for (ijk=0;ijk<n;ijk++){
    //fprintf (Outc,"\n %ld %e",ijk,tdlhs[ijk]);
    //}

    //  d_{i+1}=\tilda{d_{i+1}} + alpha.dt.v_{i+1}
    for (j=0;j<n;j++){
      lhs[j] = d[j] + alpha*dt*tdlhs[j];
    }
    
    //fprintf (Outc,"\n\n\n\n\n kontrola obnovenych promennych");
    //for (ijk=0;ijk<n;ijk++){
    //fprintf (Outc,"\n %ld %e",ijk,lhs[ijk]);
    //}
    
    solution_correction ();
    approximationc ();
    
    //fprintf (stdout,"\n iterace cislo %ld",i);

    internal_gforces (lcid,fi);

    //fprintf (Outc,"\n\n kontrola vnitrnich toku\n");
    //for (j=0;j<n;j++){
    //fprintf (Outc,"\n %ld  %30.20f",j,fi[j]);
    //}
    
    //  vector of unbalanced fluxes
    if (Cp->restype == fluxesc){
      for (k=0;k<n;k++){
	fb[k]=fi[k];
      }
    }
    if (Cp->restype == lrhsc){
      D0mat->gmxv (tdlhs,fi);
      for (k=0;k<n;k++){
	fb[k] = rhs[k] - p[k] - fi[k];
      }
    }

    norfb = ss (fb,fb,n);
    
    if (Mesprc==1)  fprintf (stdout,"\n%e %e",norfb,err);
    
    if (norfb<err){
      Cp->time = Cp->timecon.newtime ();
      dt = Cp->timecon.actualbacktimeincr ();
      i++;
      
      Mp->time = Cp->time;
      Mp->timecon.take_values (Cp->timecon);
      //Mp->timecon.seconds_days ();
      
      Tp->time = Cp->time;
      Tp->timecont.take_values (Cp->timecon);
      
      print_step(0,i,Cp->time,NULL);//zatim??!!
      print_flush();
      print_stept(0,i,Cp->time,NULL);//zatim??!!
      print_flusht();
      continue;
    }
    
    //  inner iteration loop
    for (j=0;j<ini;j++){
      
      if (Cp->fcsolv == fullnewtonc){
	//  stiffness and conductivity matrices
	zero_order_matrix (lcid);
	
	//  capacity matrix
	first_order_matrix (lcid);
	
	//  auxiliary vector  K (d+(1-alpha)*dt*v)
	D0mat->gmxv (d,p);
	
	//  matrix of the system of equations
	//  C + alpha.dt.K
	D0mat->scalgm (dt*alpha);
	D0mat->addgm (1.0,*D1mat);
      }
      
      //D0mat->solve_system (Gtt,p,fb);
      Cp->ssle->solve_system (Gtt,D0mat,p,fb,Outt);

      for (k=0;k<n;k++){
        tdlhs[k]+=p[k];
        lhs[k]+=alpha*dt*p[k];
      }
      
      solution_correction ();
      approximationc ();
      internal_gforces (lcid,fi);
      
      //fprintf (Outc,"\n\n kontrola vnitrnich toku ve vnitrim cyklu\n");
      //for (jj=0;jj<n;jj++){
      //fprintf (Outc,"\n %ld  %30.20f",jj,fi[jj]);
      //}
      
      //  vector of unbalanced fluxes
      if (Cp->restype == fluxesc){
	for (k=0;k<n;k++){
	  fb[k]=fi[k];
	}
      }
      if (Cp->restype == lrhsc){
	D0mat->gmxv (tdlhs,fi);
	for (k=0;k<n;k++){
	  fb[k] = rhs[k] - p[k] - fi[k];
	}
      }
      
      norfb = ss (fb,fb,n);
      
      if (Mesprc==1)  fprintf (stdout,"\n iterace %ld   chyba %e",j,norfb);
      
      if (norfb<err){
        break;
      }
    }
    


    //  new time and new time increment
    Cp->time = Cp->timecon.newtime ();
    dt = Cp->timecon.actualbacktimeincr ();
    i++;
    
    Mp->time = Cp->time;
    Mp->timecon.take_values (Cp->timecon);
    //Mp->timecon.seconds_days ();
    
    Tp->time = Cp->time;
    Tp->timecont.take_values (Cp->timecon);
    
    print_step(0,i,Cp->time,NULL);//zatim??!!
    print_flush();
    print_stept(0,i,Cp->time,NULL);//zatim??!!
    print_flusht();
    
    fprintf (Outc,"\n\n\n posun konce tyce d %f",lhs[Gtm->gnodes[4].cn[0]]);

  }while(Cp->time<end_time);
  
  
  delete [] fi;
  delete [] fb;
  delete [] p;
  delete [] arhs;
  delete [] d;
}


void vector_assemb (double *c,double *m,double *t)
{
  long i,j;
  
  for (i=0;i<Gtm->ndof;i++){
    j=Gtm->cngtopcorr[i];
    c[j]=m[i];
  }
  for (i=0;i<Gtt->ndof;i++){
    j=Gtt->cngtopcorr[i];
    c[j]=t[i];
  }

}

void vector_decomp (double *c,double *m,double *t)
{
  long i,j;
  
  for (i=0;i<Gtm->ndof;i++){
    j=Gtm->cngtopcorr[i];
    m[i]=c[j];
  }
  for (i=0;i<Gtt->ndof;i++){
    j=Gtt->cngtopcorr[i];
    t[i]=c[j];
  }

}
