/*
    File:             onemediumc.cpp
    Author:           Tomas Krejci
    Purpose:          computes conductivity and capacity matrices in a material point for coupled three media trasport with mechanics
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "aliast.h"
#include "onemediumc.h"
#include "coupmatu.h"
#include "coupmatl.h"
#include "globalc.h"
#include "global.h"
#include "globalt.h"
#include "isotrmatc.h"
#include "intpoints.h"
#include "consol_awf1.h"

medc1::medc1()
{  
  /* scale_t = Tp->scale1;
     scale_u = Mp->scale;
  */
  //not completed
  scale_t = 1.0;
  scale_u = 1.0;
}
medc1::~medc1()
{}


/**
   function computes conductivity matrix D in a material point for one medium transfer
   for upper block
   
   @param d   - conductivity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcond_u (matrix &d,long ri,long ci,long ipp)
{
  long i,m;
  double k;
  m = d.m;
  
  i = Cmu->ip[ipp].idm;
  
  switch (Cp->mednam){//names of transported media
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      matrix sd(d.m,d.n);
      //t  = Cmu->ip[ipp].av[0]*scale_t;//scaling
      k = Cmu->itrmc[i].get_alpha();//*scale_u;//scaling
      k = -1.0*k;
      
      switch (m){
      case 1:{//1D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	break;
      }
      case 3:{//2D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	sd[1][0] = k;
	sd[2][0] = 0.0;  
	break;
      }
      case 6:{//3D
	fillm(0.0,sd);
	
	sd[0][0]=k;  
	sd[1][0]=k;   
	sd[2][0]=k;
	sd[3][0]=0.0;   
	sd[4][0]=0.0;  
	sd[5][0]=0.0;
	break;
      }
      default:{
	print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
      }
      }
      matrix s(d.m,d.m);
      
      Cmu->matstiff (s,ipp);//only for temperature
      mxm(s,sd,d);
      
      destrm (s); destrm (sd);
      
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  case mech_water:{
    switch (Cmu->ip[ipp].tm){//material type
    case sejtkrc:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      k = Cmu->sejtkrmc[i].get_kuw(pw);//*scale_u;//scaling
      break;
    }
    case consolawf1c:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      k = Cmu->consol_awf1c[i].get_kuw(pw);//*scale_u;//scaling
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    
    switch (m){
    case 1:{//1D
      d[0][0] = k;
      break;
    }
    case 3:{//2D
      d[0][0] = k;
      d[1][0] = k;
      d[2][0] = 0.0;  
      break;
    }
    case 6:{//3D
      d[0][0]=k;  
      d[1][0]=k;   
      d[2][0]=k;
      d[3][0]=0.0;   
      d[4][0]=0.0;  
      d[5][0]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}




/**
   function computes capacity matrix D in a material point for one medium transfer
   for upper block
   
   @param d   - cpapcity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcap_u (matrix &d,long ri,long ci,long ipp)
{
  long i,m;
  double c;
  m = d.m;
  
  i = Cmu->ip[ipp].idm;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      //t  = Cmu->ip[ipp].av[0]*scale_t;//scaling
      
      c = 0.0;
      
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      c = Cmu->sejtkrmc[i].get_capuw(pw);//*scale_u;//scaling
      break;
    }
      
    case consolawf1c:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      c = Cmu->consol_awf1c[i].get_capuw(pw);//*scale_u;//scaling
      break;
    }

    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }  
    
    switch (m){
    case 1:{//1D
      fillm(0.0,d);
      
      d[0][0] = c;
      break;
    }
    case 3:{//2D
      fillm(0.0,d);
      
      d[0][0] = c;
      d[1][0] = c;
      d[2][0] = 0.0;  
      break;
    }
    case 6:{//3D
      fillm(0.0,d);
      
      d[0][0]=c;  
      d[1][0]=c;   
      d[2][0]=c;
      d[3][0]=0.0;   
      d[4][0]=0.0;  
      d[5][0]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}


/**
   function computes conductivity matrix D in a material point for one medium transfer
   for lower block
   
   @param d   - conductivity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcond_l (matrix &d,long ri,long ci,long ipp)
{
  long i,n;
  double k;
  n = d.n;
  
  i = Cml->ip[ipp].idm;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cml->ip[ipp].tm){//material type
    case isotransmatc:{
      //t  = Cml->ip[ipp].av[0]*scale_t;//scaling
      k = 0.0;
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      k = Cmu->sejtkrmc[i].get_kwu(pw);//*scale_u;//scaling
      break;
    }
    case consolawf1c:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      k = Cmu->consol_awf1c[i].get_kwu(pw);//*scale_u;//scaling
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    
    switch (n){
    case 1:{//1D
      fillm(0.0,d);
      
      d[0][0] = k;
      break;
    }
    case 3:{//2D
      fillm(0.0,d);
      
      d[0][0] = k;
      d[0][1] = k;
      d[0][2] = 0.0;  
      break;
    }
    case 6:{//3D
      fillm(0.0,d);
      
      d[0][0]=k;  
      d[0][1]=k;   
      d[0][2]=k;
      d[0][3]=0.0;   
      d[0][4]=0.0;  
      d[0][5]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}




/**
   function computes capacity matrix D in a material point for one medium transfer
   for lower block
   
   @param d   - cpapcity %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::matcap_l (matrix &d,long ri,long ci,long ipp)
{
  long i,n;
  double c;
  n = d.n;
  
  i = Cml->ip[ipp].idm;

  switch (Cp->mednam){//names of transported media
  case mech_water:
  case mech_heat:{
    
    switch (Cml->ip[ipp].tm){//material type
    case isotransmatc:{
      //t  = Cml->ip[ipp].av[0]*scale_t;//scaling
      
      c = 0.0;
      break;
    }
    case sejtkrc:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      c = Cmu->sejtkrmc[i].get_capwu(pw);//*scale_u;//scaling
      break;
    }
    case consolawf1c:{
      double pw;
      
      pw = Cmu->ip[ipp].av[0];
      c = Cmu->consol_awf1c[i].get_capwu(pw);//*scale_u;//scaling
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    } 
        
    switch (n){
    case 1:{//1D
      fillm(0.0,d);
      
      d[0][0] = c;
      break;
    }
    case 3:{//2D
      fillm(0.0,d);
      
      d[0][0] = c;
      d[0][1] = c;
      d[0][2] = 0.0;  
      break;
    }
    case 6:{//3D
      fillm(0.0,d);
      
      d[0][0]=c;  
      d[0][1]=c;   
      d[0][2]=c;
      d[0][3]=0.0;   
      d[0][4]=0.0;  
      d[0][5]=0.0;
      break;
    }
    default:{
      print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}



/**
   function computes right-hand-side matrix D in a material point for one medium transfer
   for upper block
   
   @param d   - right-hand-side %matrix
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point

*/
void medc1::rhs_u (matrix &d,long ri,long ci,long ipp)
{
  long i,m;
  double k;
  m = d.m;
  
  i = Cmu->ip[ipp].idm;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:{
    switch (Cmu->ip[ipp].tm){//material type
    case consolawf1c:
    case sejtkrc:{
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }	  
    break;
  }
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      matrix sd(d.m,d.n);
      //t  = Cmu->ip[ipp].av[0]*scale_t;//scaling
      
      k = Cmu->itrmc[i].get_alpha();//*scale_u;//scaling
      k = -1.0*k;
      
      switch (m){
      case 1:{//1D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	break;
      }
      case 3:{//2D
	fillm(0.0,sd);
	
	sd[0][0] = k;
	sd[1][0] = k;
	sd[2][0] = 0.0;  
	break;
      }
      case 6:{//3D
	fillm(0.0,sd);
	
	sd[0][0]=k;  
	sd[1][0]=k;   
	sd[2][0]=k;
	sd[3][0]=0.0;   
	sd[4][0]=0.0;  
	sd[5][0]=0.0;
	break;
      }
      default:{
	print_err("\n Unknown number of components of stress tensor is required in ", __FILE__, __LINE__, __func__);
      }
      }
      matrix s(d.m,d.m);
      
      Cmu->matstiff (s,ipp);//only for temperature
      mxm(s,sd,d);
      
      destrm (s); destrm (sd);
      
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media name is required in ", __FILE__, __LINE__, __func__);
  }
  }
}



/**
   function computes volume part of right-hand
   in the required integration point
   
   @param d - right-hand side %matrix of a material
   @param ri  - row index
   @param ci  - column index
   @param ipp - number of integration point
*/
void medc1::rhs_volume (matrix &d,long ri, long ci,long ipp)
{
  long i;
  
  //  material id
  i = Cmu->ip[ipp].idm;
  
  switch (Cmu->ip[ipp].tm){
  case sejtkrc:
    Cmu->sejtkrmc[i].rhs_volume(d,ri,ci,ipp);
    break;
  case consolawf1c:
    Cmu->consol_awf1c[i].rhs_volume(d,ri,ci,ipp);
    break;
  case isotransmatc:
    break;
  default:{
    print_err ("unknown material type is required",__FILE__,__LINE__,__func__);
  }
  }
}




/**
   function computes initial values in a material point for one medium transfer
   for upper block
   
   @param ipp - number of integration point

*/
void medc1::initvalues (long ipp)
{
  long i,nc;

  nc = Cmu->ip[ipp].nmcomp;

  switch (Cp->mednam){//names of transported media
  case mech_water:{
    
    //null eigstrains
    for(i=0;i<nc;i++)
      Cmu->eigstrains[ipp][i] = 0;
    
    Cmu->inittempr[ipp] = Cmu->ip[ipp].av[0];//inital water pressure - docasne??!!

    break;
  }
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      
      //null temprstrains
      for(i=0;i<nc;i++)
	Cmu->tempstrains[ipp][i] = 0;

      Cmu->inittempr[ipp] = Cmu->ip[ipp].av[0];//inital temperature

      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media type is required in ", __FILE__, __LINE__, __func__);
  }
  }
}


/**
   function computes update values in a material point for one medium transfer
   for upper block
   
   @param ipp - number of integration point

*/
void medc1::updatevalues (long ipp)
{
  long i,nc;

  nc = Cmu->ip[ipp].nmcomp;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
    switch (Cmu->ip[ipp].tm){//material type
    case sejtkrc:     
      break;
    case consolawf1c:{      
      
      //null temprstrains
      for(i=0;i<nc;i++)
	Cmu->eigstrains[ipp][i] = 0;
      
      Cmu->tempr[ipp] = Cmu->ip[ipp].av[0];//actual water pressure - docasne??!!
      
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      
      //null temprstrains
      for(i=0;i<nc;i++)
	Cmu->tempstrains[ipp][i] = 0;

      Cmu->tempr[ipp] = Cmu->ip[ipp].av[0];//actual temperature
      
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media type is required in ", __FILE__, __LINE__, __func__);
  }
  }
}

/**
   function computes temperature strains in a material point for one medium transfer
   for upper block
   
   @param ipp - number of integration point

*/
void medc1::tempstrains (long ipp)
{
  long i,nc;
  double dt,alpha;
  strastrestate sss;
  
  nc = Cmu->ip[ipp].nmcomp;

  //  type of strain/stress state
  sss = Cmu->ip[ipp].mssst;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
    
    switch (Cmu->ip[ipp].tm){//material type
    case sejtkrc:
    case consolawf1c:{
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  case mech_heat:{
    
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      

      //null temprstrains
      for(i=0;i<nc;i++)
	Cmu->tempstrains[ipp][i] = 0;
      
      //  change of temperature
      dt = Cmu->tempr[ipp]-Cmu->inittempr[ipp];

      i = Cmu->ip[ipp].idm;
      alpha = Cmu->itrmc[i].get_alpha();
      
      switch (sss){
      case bar:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	break;
      }
      case plbeam:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=0.0;
	Cmu->tempstrains[ipp][2]=0.0;
	break;
      }
      case planestress:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=alpha*dt;
	Cmu->tempstrains[ipp][3]=alpha*dt;
	break;
      }
      case planestrain:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=alpha*dt;
	Cmu->tempstrains[ipp][3]=alpha*dt;
	break;
      }
      case axisymm:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=alpha*dt;
	Cmu->tempstrains[ipp][2]=alpha*dt;
	break;
      }
      case spacestress:{
	Cmu->tempstrains[ipp][0]=alpha*dt;
	Cmu->tempstrains[ipp][1]=alpha*dt;
	Cmu->tempstrains[ipp][2]=alpha*dt;
	Cmu->tempstrains[ipp][3]=0.0;
	Cmu->tempstrains[ipp][4]=0.0;
	Cmu->tempstrains[ipp][5]=0.0;    
	break;
      }
      default:
	print_err("unknown strain/stress state is required", __FILE__, __LINE__, __func__);
      }
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media type is required in ", __FILE__, __LINE__, __func__);
  }
  }
}

/**
   function computes eigen strains (other strains from nonmechanical loading) in a material point for one medium transfer
   for upper block
   
   @param ipp - number of integration point

*/
void medc1::eigstrains (long ipp)
{
  long i,nc;
  double eigstr,pw,dpw,nu;
  strastrestate sss;

  nc = Cmu->ip[ipp].nmcomp;
  
  //  type of strain/stress state
  sss = Cmu->ip[ipp].mssst;
  
  switch (Cp->mednam){//names of transported media
  case mech_water:
    
    switch (Cmu->ip[ipp].tm){//material type
    case sejtkrc:{
      break;
    }
    case consolawf1c:{
      //null eigstrains
      for(i=0;i<nc;i++)
	Cmu->eigstrains[ipp][i] = 0;

      pw = Cmu->ip[ipp].av[0];

      //  total increment of pore water pressure - temporarily stored in tempr and inittempr
      //dpw = Cmu->tempr[ipp]-Cmu->inittempr[ipp];
      //this is the same:
      dpw = pw - Cmu->inittempr[ipp];

      i = Cmu->ip[ipp].idm;
      eigstr = Cmu->consol_awf1c[i].get_eigstr(pw);
      nu = Cmu->consol_awf1c[i].nu;
      
      if(Cmu->nonmechtransq != NULL){
	Cmu->nonmechtransq[ipp][1] = dpw*eigstr;
	Cmu->nonmechtransq[ipp][2] = pw*eigstr;
      }

      switch (sss){
      case bar:{
	Cmu->eigstrains[ipp][0]=eigstr*dpw;
	break;
      }
      case plbeam:{
	Cmu->eigstrains[ipp][0]=eigstr*dpw;
	Cmu->eigstrains[ipp][1]=0.0;
	Cmu->eigstrains[ipp][2]=0.0;
	break;
      }
      case planestress:{
	Cmu->eigstrains[ipp][0]=eigstr*dpw*(1.0 - 2.0*nu);//??!!
	Cmu->eigstrains[ipp][1]=eigstr*dpw*(1.0 - 2.0*nu);//??!!
	Cmu->eigstrains[ipp][3]=eigstr*dpw*(1.0 - 2.0*nu);//??!!
	break;
      }
      case planestrain:{
	Cmu->eigstrains[ipp][0]=eigstr*dpw*(1.0 - 2.0*nu);//??!!
	Cmu->eigstrains[ipp][1]=eigstr*dpw*(1.0 - 2.0*nu);//??!!
	Cmu->eigstrains[ipp][3]=eigstr*dpw*(1.0 - 2.0*nu);//??!!
	break;
      }
      case axisymm:{
	Cmu->eigstrains[ipp][0]=eigstr*dpw*(1.0 - 2.0*nu);//??!!
	Cmu->eigstrains[ipp][1]=eigstr*dpw*(1.0 - 2.0*nu);//??!!
	Cmu->eigstrains[ipp][2]=eigstr*dpw*(1.0 - 2.0*nu);//??!!
	break;
      }
      case spacestress:{
	Cmu->eigstrains[ipp][0]=eigstr*dpw*(1.0 - 2.0*nu);
	Cmu->eigstrains[ipp][1]=eigstr*dpw*(1.0 - 2.0*nu);
	Cmu->eigstrains[ipp][2]=eigstr*dpw*(1.0 - 2.0*nu);
	Cmu->eigstrains[ipp][3]=0.0;
	Cmu->eigstrains[ipp][4]=0.0;
	Cmu->eigstrains[ipp][5]=0.0;    
	break;
      }
      default:
	print_err("unknown strain/stress state is required", __FILE__, __LINE__, __func__);
      }
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  case mech_heat:{
    switch (Cmu->ip[ipp].tm){//material type
    case isotransmatc:{
      break;
    }
    default:{
      print_err("\n Unknown material type is required in ", __FILE__, __LINE__, __func__);
    }
    }
    break;
  }
  default:{
    print_err("\n Unknown media type is required in ", __FILE__, __LINE__, __func__);
  }
  }
}
